import json
import getpass
import argparse
import subprocess
from .core import *

def get_branch_name():
    try:
        branch_name = subprocess.check_output(
            ['git', 'symbolic-ref', '--short', '-q', 'HEAD']).decode("utf-8")
        branch_name = branch_name.strip()
        return branch_name
    except subprocess.CalledProcessError as e:
        out_bytes = e.output
        code = e.returncode
        return None

def main():
    parser = argparse.ArgumentParser(description='Code sync command line application.')
    parser.add_argument('--server-config', type=str, default="~/.config/csync/servers.json")
    parser.add_argument('--server', '-s', type=str, default=None)
    parser.add_argument('--username', '-u', type=str, default=None)
    parser.add_argument('--keyfile', '-k', type=str, default="~/.ssh/id_rsa")
    parser.add_argument('--local-path', '-l', type=str, default=".")
    parser.add_argument('--remote-path', '-r', type=str, default=None)
    parser.add_argument('--remote-port', type=int, default=22)
    parser.add_argument('--no-ignore-hidden-file', action="store_true", default=False)
    parser.add_argument('--ignore-filetypes', type=str, default="['.pyc']")
    parser.add_argument('--file-limited-size', type=str, default="1024**2")
    parser.add_argument('--no-branch-name', action="store_true", default=False)
    
    args = parser.parse_args()

    ignore_filetypes = eval(args.ignore_filetypes)
    file_limited_size = eval(args.file_limited_size)

    with open(os.path.expanduser(args.server_config), "r") as f:
        server_config = json.load(f)
    servers = server_config["servers"]
    
    try:
        remote_ip  = servers[args.server]
    except Exception:
        raise ValueError(f"server can not be {args.server}")

    ssh, remote_home = connect_to(remote_ip=remote_ip, remote_port=args.remote_port,
    username=args.username or getpass.getuser(), keyfile=os.path.expanduser(args.keyfile))

    local_path = os.path.abspath(args.local_path)
    if args.remote_path is None:
        relpath = os.path.relpath(local_path, os.path.expanduser("~"))
        remote_path = os.path.join(remote_home, relpath)
    else:
        remote_path = args.remote_path

    if not args.no_branch_name:
        remote_path = os.path.join(remote_path, get_branch_name())
    
    sync(local_path, remote_ip, ssh, remote_path,  
    not args.no_ignore_hidden_file, ignore_filetypes,file_limited_size)


if __name__ == "__main__":
    main()
    


from setuptools import setup

setup(
    name = "pycodesync",
    version = "0.1.0",
    description="Code sync command line application.",
    author="chenyaofo",
    author_email = "chenyaofo@gmail.com",
    packages = ["pycodesync"],
    entry_points = {
        "console_scripts": [
            "csync = pycodesync.__main__:main"
        ]
    },
    install_requires=["tqdm","paramiko"]
)


import os
import tqdm
import paramiko

remote_dirs_cache = set()

def is_ignore_filetype(filename: str, ignore_filetypes):
    for ignore_filetype in ignore_filetypes:
        if filename.endswith(ignore_filetype):
            return True
    return False

def sftp_exists(sftp, path, cache=remote_dirs_cache):
    if path in cache:
        return True
    try:
        sftp.stat(path)
        cache.add(path)
        return True
    except FileNotFoundError:
        return False

def get_root_dirs(dir):
    rev = []
    p = dir
    while p != "/":
        rev.append(p)
        p = os.path.dirname(p)
    return reversed(rev)

def sftp_mkdirs(sftp, path):
    for p in get_root_dirs(path):
        if not sftp_exists(sftp, p):
            sftp.mkdir(p)

def get_remote_home(ssh):
    stdin, stdout, stderr = ssh.exec_command('echo $HOME')
    remote_home = stdout.readlines()[0].strip()
    return remote_home

def connect_to(remote_ip, remote_port, username, keyfile):
    ssh = paramiko.SSHClient()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    ssh.connect(remote_ip, remote_port, username, key_filename=keyfile)

    remote_home = get_remote_home(ssh)
    
    return ssh, remote_home

def sync(local_path, remote_ip, ssh, remote_path, 
         ignore_hidden_file=True, ignore_filetypes=[],
         file_limited_size=float("inf")):
    sftp = paramiko.SFTPClient.from_transport(ssh.get_transport())

    candidates = []
    for root, dirs, files in os.walk(local_path):
        if ignore_hidden_file and root.startswith("."):
            continue

        for f in files:
            filename = os.path.join(root, f)
            if ignore_hidden_file and f.startswith("."):
                continue
            if is_ignore_filetype(filename, ignore_filetypes):
                continue
            if os.path.getsize(filename) < file_limited_size:
                candidates.append(filename)
    
    print(f"local:{local_path} -> {remote_ip}:{remote_path}")

    bar_format = "{percentage:3.0f}% |{bar}| {n_fmt}/{total_fmt} [{elapsed}<{remaining}]"

    for p in tqdm.tqdm(candidates, bar_format=bar_format):
        relpath = os.path.relpath(p, local_path)
        rp = os.path.join(remote_path, relpath)
        sftp_mkdirs(sftp, os.path.dirname(rp))
        sftp.put(p, rp)